import { __awaiter } from 'tslib';
import { ɵɵdefineInjectable, ɵɵinject, Injectable } from '@angular/core';
import { DcSccBaseFactory, DcSccBaseValidators, DcScc } from 'dc-scc';
import { DcEasApi } from 'dc-eas-api';
import { DcDdsObjectModelV1 } from 'dc-dds-api';

class BaseFactory extends DcSccBaseFactory {
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            return reqObj;
        });
    }
    clearCache(obj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[obj.getGuid()] != "undefined") {
                delete this._s[obj.getGuid()];
            }
            return this;
        });
    }
    getByGuids(guids, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var utilObj = this.getService().getDcScc().getUtilities().getFactories().getAsyncGuids();
            return yield utilObj.getByGuids(guids, this, "getByGuid", cbObj, cbMethod);
        });
    }
}

class Connection extends BaseFactory {
    constructor() {
        super(...arguments);
        this._wsHost = "dc-wss-nlb.datacamo.net";
        this._wsPort = 28103;
        this._wsSsl = true;
        this._isReconn = false;
        this._wrkObj = null;
    }
    setConfig(host, port, ssl = true) {
        this._wsHost = host;
        this._wsPort = port;
        this._wsSsl = ssl;
        return this;
    }
    getClient() {
        return __awaiter(this, void 0, void 0, function* () {
            var wrkObj = yield this.getWorker();
            return wrkObj.getClient();
        });
    }
    getWorker() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._wrkObj === null) {
                try {
                    this._wrkObj = this.getService().getDcScc().getWorkers().getV3(this._wsHost, this._wsPort, this._wsSsl);
                    yield this._wrkObj.initialize();
                    var cObj = this._wrkObj.getClient();
                    var reqObj = cObj.newMessage();
                    reqObj.setL1("Authentication").setL2("Get").setL3("Salt");
                    var salt = yield reqObj.getResponse(true);
                    var reqObj = cObj.newMessage();
                    reqObj.setL1("Authentication").setL2("Validate").setL3("ByEAS").setL4("EphemeralKey");
                    var hash = yield this.getMtm().get("utilities").getHashing().getSHA().get256("RAS--" + salt);
                    var ephKey = yield this.getMtm().get("DcEasApi").getEphemeralKeys().newReadUserPublic();
                    yield ephKey.signRequest(reqObj, hash);
                    var sessObj = yield reqObj.getResponse(true);
                    if (typeof sessObj["session-id"] != "undefined"
                        && typeof sessObj["session-key"] != "undefined") {
                        cObj.setSession(sessObj["session-id"], sessObj["session-key"]);
                        this._wrkObj.setTermCb(this, "reconnect");
                    }
                    else {
                        throw this.getMtm().newException("Server authentication returned an invalid session");
                    }
                }
                catch (errObj) {
                    this._wrkObj = null;
                    throw errObj;
                }
            }
            return this._wrkObj;
        });
    }
    reconnect(wrkObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._isReconn === false) {
                this._isReconn = true;
                this.logInfo("RAS Reconnecting");
                try {
                    try {
                        this._wrkObj = null;
                        yield this.getWorker();
                        this._isReconn = false;
                        this.logInfo("RAS Reconnect success");
                    }
                    catch (errObj) {
                        throw this.getService().getMtm().newException("RAS Reconnect failed");
                    }
                }
                catch (errObj) {
                    this._isReconn = false;
                    this.callException(errObj);
                }
            }
        });
    }
}

class Base extends DcDdsObjectModelV1 {
    constructor() {
        super(...arguments);
        this._guid = null;
    }
    baseConstruct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            this.setExceptionCb(this.getService().getLogging(), "exception");
            this._guid = apiObj.guid;
            if (typeof apiObj["ddsId"] != "undefined") {
                var ddsObj = yield this.getService().getDdsClient();
                this.setConfig(ddsObj, apiObj.ddsId, apiObj.ddsAuth);
                var storeObj = yield this.pubStore();
                yield storeObj.initialize();
            }
        });
    }
    getGuid() {
        return this._guid;
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            return reqObj;
        });
    }
}

class Network extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getNs() {
        return this.getPubKey("ns", true);
    }
    setNs(permObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (permObj.getNs() !== this.getNs()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Networks").setL2("Set").setL3("Namespace");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("namespace", permObj.getNs());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    setDisplayName(name) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getDisplayName() !== name) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Networks").setL2("Set").setL3("DisplayName");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("name", name);
                var waitUp = this.getPubUpdateWait();
                try {
                    yield reqObj.getResponse(true);
                    yield waitUp;
                }
                catch (errObj) {
                    this.pubUpdateClear(waitUp);
                    throw errObj;
                }
            }
            return this;
        });
    }
    getWiredRejectVlan() {
        return this.getPubKey("wiredRejectVlan", true);
    }
    setWiredRejectVlan(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getWiredRejectVlan() !== id) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Networks").setL2("Set").setL3("WiredRejectVlan");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("id", id);
                var waitUp = this.getPubUpdateWait();
                try {
                    yield reqObj.getResponse(true);
                    yield waitUp;
                }
                catch (errObj) {
                    this.pubUpdateClear(waitUp);
                    throw errObj;
                }
            }
            return this;
        });
    }
    getWirelessRejectVlan() {
        return this.getPubKey("wlanRejectVlan", true);
    }
    setWirelessRejectVlan(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getWirelessRejectVlan() !== id) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Networks").setL2("Set").setL3("WirelessRejectVlan");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("id", id);
                var waitUp = this.getPubUpdateWait();
                try {
                    yield reqObj.getResponse(true);
                    yield waitUp;
                }
                catch (errObj) {
                    this.pubUpdateClear(waitUp);
                    throw errObj;
                }
            }
            return this;
        });
    }
    getNasDevices(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNasDevices().getByNetwork(this, cbObj, cbMethod);
        });
    }
    getUserMaps(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUserNetMaps().getByNetwork(this, cbObj, cbMethod);
        });
    }
    clearUserMap(mapObj) {
        return __awaiter(this, void 0, void 0, function* () {
            var netObj = yield mapObj.getNetwork();
            if (netObj.getGuid() !== this.getGuid()) {
                throw this.getMtm().newException("Cannot delete, user map is not mapped");
            }
            return this;
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getNetworks().clearCache(this);
    }
}

class Networks extends BaseFactory {
    newObj() {
        var rObj = new Network();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getBySession(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            //get all networks this user has access to
            var reqObj = yield this.newRequest();
            reqObj.setL1("Networks").setL2("Get").setL3("BySession");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Networks").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class NasDevice extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getNs() {
        return this.getPubKey("ns", true);
    }
    setNs(permObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (permObj.getNs() !== this.getNs()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("NasDevices").setL2("Set").setL3("Namespace");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("usm-ns-guid", permObj.getGuid());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    setDisplayName(name) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (name != this.getDisplayName()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("NasDevices").setL2("Set").setL3("DisplayName");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("name", name);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getNetwork(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNetworks().getByGuid(this.getPubKey("networkGuid", true), cbObj, cbMethod);
        });
    }
    setNetwork(netObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var curObj = yield this.getNetwork();
                if (netObj.getGuid() !== curObj.getGuid()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("NasDevices").setL2("Set").setL3("Network");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("net-guid", netObj.getGuid());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getNasDevices().clearCache(this);
    }
}

class NasDevices extends BaseFactory {
    newObj() {
        var rObj = new NasDevice();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByNetwork(netObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("NasDevices").setL2("Get").setL3("ByNetwork");
            reqObj.addReq("guid", netObj.getGuid());
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("NasDevices").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class User extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getUsername() {
        return this.getPubKey("username", true);
    }
    setUsername(val) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (val != this.getUsername()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Users").setL2("Set").setL3("Username");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("username", val);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    setPassword(val) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Users").setL2("Set").setL3("Password");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("password", val);
                yield reqObj.getResponse(true);
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getNs() {
        return this.getPubKey("ns", true);
    }
    setNs(permObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (permObj.getNs() !== this.getNs()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Users").setL2("Set").setL3("Namespace");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("namespace", permObj.getNs());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getNote() {
        return this.getPubKey("note", true);
    }
    setNote(val) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (val != this.getNote()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Users").setL2("Set").setL3("Note");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("note", val);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getNetworkMaps(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUserNetMaps().getByUser(this, cbObj, cbMethod);
        });
    }
    clearNetworkMap(mapObj) {
        return __awaiter(this, void 0, void 0, function* () {
            var userObj = yield mapObj.getUser();
            if (userObj.getGuid() !== this.getGuid()) {
                throw this.getMtm().newException("Cannot delete, user map is not mapped");
            }
            return this;
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Users").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getUsers().clearCache(this);
    }
}

class Users extends BaseFactory {
    newObj() {
        var rObj = new User();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getBySession(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            //get all networks this user has access to
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("BySession");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                if (typeof this._s[apiObj.guid] === "undefined") {
                    this._s[apiObj.guid] = this.newObj();
                    this._s[apiObj.guid].construct(apiObj);
                }
                yield this._s[apiObj.guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[apiObj.guid]);
                }
                return this._s[apiObj.guid];
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    add(username, password, usmNsObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Add").setL3("ByUserPass");
            reqObj.addReq("username", username);
            reqObj.addReq("password", password);
            reqObj.addReq("usm-ns-guid", usmNsObj.getGuid());
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
}

class Permission extends Base {
    constructor() {
        super(...arguments);
        this._ns = null;
        this._role = null;
    }
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this._ns = apiObj.ns;
            this._role = apiObj.role;
            this.setInit();
            return this;
        });
    }
    getNs() {
        return this._ns;
    }
    getRole() {
        return this._role;
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getPermissions().clearCache(this);
    }
}

class Permissions extends BaseFactory {
    newObj() {
        var rObj = new Permission();
        rObj.setService(this.getService());
        return rObj;
    }
    getBySession(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            //get all networks this user has access to
            var reqObj = yield this.newRequest();
            reqObj.setL1("Permissions").setL2("Get").setL3("BySession");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Permissions").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                if (typeof this._s[apiObj.guid] === "undefined") {
                    this._s[apiObj.guid] = this.newObj();
                    this._s[apiObj.guid].construct(apiObj);
                }
                yield this._s[apiObj.guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[apiObj.guid]);
                }
                return this._s[apiObj.guid];
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class UserNetMap extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getUser(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUsers().getByGuid(this.getPubKey("userGuid", true), cbObj, cbMethod);
        });
    }
    getNetwork(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNetworks().getByGuid(this.getPubKey("networkGuid", true), cbObj, cbMethod);
        });
    }
    getAttrs(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUserNetMapAttrs().getByMap(this, cbObj, cbMethod);
        });
    }
    addAttr(typeObj, value) {
        return __awaiter(this, void 0, void 0, function* () {
            var attrObj = yield this.getAttrByType(typeObj, false);
            if (attrObj !== null) {
                throw this.getMtm().newException("Cannot add attribute exists");
            }
            var reqObj = yield this.getService().newRequest();
            reqObj.setL1("UserNetMapAttrs").setL2("Add");
            reqObj.addReq("map-guid", this.getGuid());
            reqObj.addReq("type-guid", typeObj.getGuid());
            reqObj.addReq("value", value);
            var apiObj = yield reqObj.getResponse(true);
            return yield this.getService().getUserNetMapAttrs().getByApiObj(apiObj);
        });
    }
    getAttrByType(typeObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var attrObjs = yield this.getAttrs();
            for (let attrObj of attrObjs) {
                var eType = yield attrObj.getType();
                if (eType.getGuid() === typeObj.getGuid()) {
                    return attrObj;
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("Attribute type is not mapped");
            }
            else {
                return null;
            }
        });
    }
    setAttr(typeObj, value) {
        return __awaiter(this, void 0, void 0, function* () {
            var attrObj = yield this.getAttrByType(typeObj, false);
            if (attrObj === null) {
                attrObj = yield this.addAttr(typeObj, value);
            }
            else {
                yield attrObj.setValue(value);
            }
            return attrObj;
        });
    }
    clearAttr(attrObj) {
        return __awaiter(this, void 0, void 0, function* () {
            var mapObj = yield attrObj.getMap();
            if (mapObj.getGuid() !== this.getGuid()) {
                throw this.getMtm().newException("Cannot delete, attribute is not mapped");
            }
            return this;
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var netObj = yield this.getNetwork();
                netObj.clearUserMap(this);
                var userObj = yield this.getUser();
                userObj.clearNetworkMap(this);
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("UserNetMaps").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getUserNetMaps().clearCache(this);
    }
}

class UserNetMaps extends BaseFactory {
    newObj() {
        var rObj = new UserNetMap();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByNetwork(netObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var sTime = this.getMtm().get("utilities").getTime().getMicroEpoch();
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMaps").setL2("Get").setL3("ByNetwork");
            reqObj.addReq("guid", netObj.getGuid());
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByUser(userObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var sTime = this.getMtm().get("utilities").getTime().getMicroEpoch();
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMaps").setL2("Get").setL3("ByUser");
            reqObj.addReq("guid", userObj.getGuid());
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMaps").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    add(userObj, netObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMaps").setL2("Add");
            reqObj.addReq("user-guid", userObj.getGuid());
            reqObj.addReq("net-guid", netObj.getGuid());
            try {
                var apiObj = yield reqObj.getResponse(true);
                if (typeof this._s[apiObj.guid] === "undefined") {
                    this._s[apiObj.guid] = this.newObj();
                    this._s[apiObj.guid].construct(apiObj);
                }
                yield this._s[apiObj.guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[apiObj.guid]);
                }
                return this._s[apiObj.guid];
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class UserNetMapAttr extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getMap() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUserNetMaps().getByGuid(this.getPubKey("mapGuid", true));
        });
    }
    getType() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getAttrTypes().getByGuid(this.getPubKey("typeGuid", true));
        });
    }
    setValue(val) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getValue() !== val) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("UserNetMapAttrs").setL2("Set").setL3("Value");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("value", val);
                var waitUp = this.getPubUpdateWait();
                try {
                    yield reqObj.getResponse(true);
                    yield waitUp;
                }
                catch (errObj) {
                    this.pubUpdateClear(waitUp);
                    throw errObj;
                }
            }
            return this;
        });
    }
    getValue() {
        return this.getPubKey("value", true);
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var mapObj = yield this.getMap();
                mapObj.clearAttr(this);
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("UserNetMapAttrs").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getUserNetMapAttrs().clearCache(this);
    }
}

class UserNetMapAttrs extends BaseFactory {
    newObj() {
        var rObj = new UserNetMapAttr();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByMap(netObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMapAttrs").setL2("Get").setL3("ByMap");
            reqObj.addReq("guid", netObj.getGuid());
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMapAttrs").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class AttrType extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getLabel() {
        return this.getPubKey("label", true);
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    getName() {
        return this.getPubKey("attrName", true);
    }
    getOperator() {
        return this.getPubKey("operator", true);
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getAttrTypes().clearCache(this);
    }
}

class AttrTypes extends BaseFactory {
    newObj() {
        var rObj = new AttrType();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getAll(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("AttrTypes").setL2("Get").setL3("All");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("AttrTypes").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class Session extends DcSccBaseValidators {
    constructor() {
        super(...arguments);
        this._cObj = null;
    }
    construct(cObj) {
        return __awaiter(this, void 0, void 0, function* () {
            this._cObj = cObj;
            this.setInit();
            return this;
        });
    }
    getNetworks(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNetworks().getBySession(cbObj, cbMethod);
        });
    }
    getUsers(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUsers().getBySession(cbObj, cbMethod);
        });
    }
    getPermissions(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getPermissions().getBySession(cbObj, cbMethod);
        });
    }
}

class DcRasApi {
    constructor(_dcScc, _dcEasApi) {
        this._dcScc = _dcScc;
        this._dcEasApi = _dcEasApi;
        this._connFact = null;
        this._sessObj = null;
        this._netFact = null;
        this._nasFact = null;
        this._uFact = null;
        this._permFact = null;
        this._unmFact = null;
        this._unmaFact = null;
        this._atFact = null;
        this._ddsClient = null;
        this._dcScc.getMtm().set("DcRasApi", this);
    }
    getDcScc() {
        return this._dcScc;
    }
    getMtm() {
        return this.getDcScc().getMtm();
    }
    getLogging() {
        return this.getDcScc().getLogging();
    }
    getSession() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._sessObj === null) {
                this._sessObj = new Session();
                this._sessObj.setService(this);
                this._sessObj.construct(yield this.getConnection().getClient());
                this._sessObj.setExceptionCb(this.getLogging(), "exception");
            }
            return this._sessObj.waitInit();
        });
    }
    getConnection() {
        if (this._connFact === null) {
            this._connFact = new Connection();
            this._connFact.setService(this);
            this._connFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._connFact;
    }
    getNetworks() {
        if (this._netFact === null) {
            this._netFact = new Networks();
            this._netFact.setService(this);
            this._netFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._netFact;
    }
    getNasDevices() {
        if (this._nasFact === null) {
            this._nasFact = new NasDevices();
            this._nasFact.setService(this);
            this._nasFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._nasFact;
    }
    getUserNetMaps() {
        if (this._unmFact === null) {
            this._unmFact = new UserNetMaps();
            this._unmFact.setService(this);
            this._unmFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._unmFact;
    }
    getUserNetMapAttrs() {
        if (this._unmaFact === null) {
            this._unmaFact = new UserNetMapAttrs();
            this._unmaFact.setService(this);
            this._unmaFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._unmaFact;
    }
    getAttrTypes() {
        if (this._atFact === null) {
            this._atFact = new AttrTypes();
            this._atFact.setService(this);
            this._atFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._atFact;
    }
    getUsers() {
        if (this._uFact === null) {
            this._uFact = new Users();
            this._uFact.setService(this);
            this._uFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._uFact;
    }
    getPermissions() {
        if (this._permFact === null) {
            this._permFact = new Permissions();
            this._permFact.setService(this);
            this._permFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._permFact;
    }
    getDdsClient() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._ddsClient === null) {
                var ddsHost = "dc-wss-nlb.datacamo.net";
                var ddsPort = 28106;
                var ddsSsl = true;
                this._ddsClient = this.getMtm().get("DcDdsApi").getClients().getV1(ddsHost, ddsPort, ddsSsl);
                yield this._ddsClient.initialize();
            }
            return this._ddsClient;
        });
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var cObj = yield this.getConnection().getClient();
            var cObj = yield cObj.isAuth();
            return cObj.newMessage();
        });
    }
}
DcRasApi.ɵprov = ɵɵdefineInjectable({ factory: function DcRasApi_Factory() { return new DcRasApi(ɵɵinject(DcScc), ɵɵinject(DcEasApi)); }, token: DcRasApi, providedIn: "root" });
DcRasApi.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
DcRasApi.ctorParameters = () => [
    { type: DcScc },
    { type: DcEasApi }
];

/*
 * Public API Surface of dc-ras-api
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DcRasApi };
//# sourceMappingURL=dc-ras-api.js.map
